# -*- coding: utf-8 -*-

import copy
import sys

from notebook import nbextensions, serverextensions, extensions

from . import __version__, __name__ as _pkg_name


class ToggleJupyterTensorboardApp(
        nbextensions.ToggleNBExtensionApp,
        serverextensions.ToggleServerExtensionApp):
    """App to toggle server extension jupyter_nbextensions_configurator."""

    flags = copy.deepcopy(serverextensions.ToggleServerExtensionApp.flags)
    flags['sys-prefix'] = ({
        'ToggleServerExtensionApp': {
            'sys_prefix': True,
            'user': False,
        }},
        'Use sys.prefix as the prefix for configuring extension')
    for f in ('py', 'python'):
        flags.pop(f, None)

    def parse_command_line(self, argv=None):
        """
        Overriden to check for conflicting flags
        Since notebook version doesn't do it well (or, indeed, at all)
        """
        conflicting_flags = set(['--user', '--system', '--sys-prefix'])

        if len(conflicting_flags.intersection(set(argv))) > 1:
            raise serverextensions.ArgumentConflict(
                'cannot specify more than one of user, sys_prefix, or system')
        return super(ToggleJupyterTensorboardApp,
                     self).parse_command_line(argv)

    @property
    def name(self):
        return 'jupyter_tensorboard {}'.format(
            'enable' if self._toggle_value else 'disable')

    @property
    def description(self):
        return """
{} the jupyter_tensorboard extension in config.
Usage
    jupyter tensorboard {} [--system|--sys-prefix|--user]
""".format(*(('Enable', 'enable')
             if self._toggle_value else ('Disable', 'disable')))

    def start(self):
        """Perform the App's actions as configured."""
        if self.extra_args:
            sys.exit('{} takes no extra arguments'.format(self.name))
        else:

            if self._toggle_value:
                nbextensions.install_nbextension_python(
                    _pkg_name, overwrite=True, symlink=False,
                    user=self.user, sys_prefix=self.sys_prefix, prefix=None,
                    nbextensions_dir=None, logger=None)
            else:
                nbextensions.uninstall_nbextension_python(
                    _pkg_name, user=self.user, sys_prefix=self.sys_prefix,
                    prefix=None, nbextensions_dir=None, logger=None)

            self.toggle_nbextension_python(_pkg_name)
            self.toggle_server_extension_python(_pkg_name)


class EnableJupyterTensorboardApp(
        ToggleJupyterTensorboardApp):
    """App to enable server extension jupyter_nbextensions_configurator."""
    name = 'jupyter tensorboard enable'
    _toggle_value = True


class DisableJupyterTensorboardApp(
        ToggleJupyterTensorboardApp):
    """App to disable server extension jupyter_nbextensions_configurator."""
    name = 'jupyter tensorboard disable'
    _toggle_value = False


class JupyterTensorboardApp(extensions.BaseExtensionApp):
    """Root level jupyter_nbextensions_configurator app."""

    name = 'jupyter tensorboard'
    version = __version__
    description = (
        'Enable or disable '
        'the jupyter_tensorboard extension')
    subcommands = dict(
        enable=(
            EnableJupyterTensorboardApp,
            'Enable the jupyter_tensorboard extension.'),
        disable=(
            DisableJupyterTensorboardApp,
            'Disable the jupyter_tensorboard extension.'),
    )
    examples = '\n'.join([
        'jupyter tensorboard enable'
        '  # Enable the jupyter_tensorboard extension.',
        'jupyter tensorboard disable'
        ' # Disable the jupyter_tensorboard extension.',
    ])

    def start(self):
        """Perform the App's actions as configured"""
        super(JupyterTensorboardApp, self).start()

        # The above should have called a subcommand and raised NoStart; if we
        # get here, it didn't, so we should self.log.info a message.
        subcmds = ", ".join(sorted(self.subcommands))
        sys.exit("Please supply at least one subcommand: %s" % subcmds)


main = JupyterTensorboardApp.launch_instance


if __name__ == '__main__':  # pragma: no cover
    main()



# -*- coding: utf-8 -*-

import json
import os

from tornado import web
from notebook.base.handlers import APIHandler, json_errors

from .handlers import notebook_dir


def _trim_notebook_dir(dir):
    return os.path.join("<notebook_dir>", os.path.relpath(dir, notebook_dir))


class TbRootHandler(APIHandler):

    @json_errors
    @web.authenticated
    def get(self):
        terms = [
            {
                'name': entry.name,
                'logdir': _trim_notebook_dir(entry.logdir),
                "reload_time": entry.thread.reload_time,
            } for entry in
            self.settings["tensorboard_manager"].values()
        ]
        self.finish(json.dumps(terms))

    @json_errors
    @web.authenticated
    def post(self):
        data = self.get_json_body()
        reload_interval = data.get("reload_interval", None)
        entry = (
            self.settings["tensorboard_manager"]
            .new_instance(data["logdir"], reload_interval=reload_interval)
        )
        self.finish(json.dumps({
                'name': entry.name,
                'logdir':  _trim_notebook_dir(entry.logdir),
                'reload_time': entry.thread.reload_time}))


class TbInstanceHandler(APIHandler):

    SUPPORTED_METHODS = ('GET', 'DELETE')

    @json_errors
    @web.authenticated
    def get(self, name):
        manager = self.settings["tensorboard_manager"]
        if name in manager:
            entry = manager[name]
            self.finish(json.dumps({
                'name': entry.name,
                'logdir':  _trim_notebook_dir(entry.logdir),
                'reload_time': entry.thread.reload_time}))
        else:
            raise web.HTTPError(
                404, "TensorBoard instance not found: %r" % name)

    @json_errors
    @web.authenticated
    def delete(self, name):
        manager = self.settings["tensorboard_manager"]
        if name in manager:
            manager.terminate(name, force=True)
            self.set_status(204)
            self.finish()
        else:
            raise web.HTTPError(
                404, "TensorBoard instance not found: %r" % name)



# -*- coding: utf-8 -*-

from tornado import web
from tornado.wsgi import WSGIContainer
from notebook.base.handlers import IPythonHandler
from notebook.utils import url_path_join as ujoin
from notebook.base.handlers import path_regex

notebook_dir = None


def load_jupyter_server_extension(nb_app):

    global notebook_dir
    notebook_dir = nb_app.notebook_dir

    web_app = nb_app.web_app
    base_url = web_app.settings['base_url']

    try:
        from .tensorboard_manager import manager
    except ImportError:
        nb_app.log.info("import tensorboard error, check tensorflow install")
        handlers = [
            (ujoin(
                base_url, r"/tensorboard.*"),
                TensorboardErrorHandler),
        ]
    else:
        web_app.settings["tensorboard_manager"] = manager
        from . import api_handlers

        handlers = [
            (ujoin(
                base_url, r"/tensorboard/(?P<name>\w+)%s" % path_regex),
                TensorboardHandler),
            (ujoin(
                base_url, r"/api/tensorboard"),
                api_handlers.TbRootHandler),
            (ujoin(
                base_url, r"/api/tensorboard/(?P<name>\w+)"),
                api_handlers.TbInstanceHandler),
        ]

    web_app.add_handlers('.*$', handlers)
    nb_app.log.info("jupyter_tensorboard extension loaded.")


class TensorboardHandler(IPythonHandler):

    def get(self, name, path):

        if path == "":
                uri = self.request.path + "/"
                if self.request.query:
                    uri += "?" + self.request.query
                self.redirect(uri, permanent=True)
                return

        self.request.path = (
            path if self.request.query
            else "%s?%s" % (path, self.request.query))

        manager = self.settings["tensorboard_manager"]
        if name in manager:
            tb_app = manager[name].tb_app
            WSGIContainer(tb_app)(self.request)
        else:
            raise web.HTTPError(404)


class TensorboardErrorHandler(IPythonHandler):
    pass
